import argparse
import glob
import logging
import os
import sys
from palmve import loader, xml, sqlite


def main():
    logging.basicConfig(level=logging.INFO)
    error = False

    for f in glob.glob('data/pools/*.txt'):
        logging.info('parsing %s', f)
        try:
            e = loader.TextLoader.from_plain_text(f)
            logging.info('loaded %d questions from %s', e.total_questions(), f)

            # We write this out in XML format because we keep the XML in the
            # repository, and it's way easier to diff and compare when there
            # are changes or new pools than the SQL is, even with sqldiff.
            x = os.path.splitext(f)[0] + '.xml'
            with open(x, 'wb') as xf:
                xf.write(xml.ElementToXml(e))

            s = os.path.splitext(f)[0] + '.sqlite3'
            if os.path.isfile(s):
                os.remove(s)
            sqlite.ElementToDb(e, s)

        except loader.ParseError as p:
            logging.error('parse error in %s:%d: %s (%s)',
                    f, p.line_nr, p.error, repr(p.line))
            error = True

    return 1 if error else 0


if __name__ == '__main__':
    sys.exit(main())
