"""This module contains code to write out an element as XML format. In the
prior C++ incantation of this code, there was also a path for reading from XML.
I have not brought that over with the python. Things are much simpler, and we
don't support palm pilot DBs anymore."""


from lxml import etree
from palmve import utils


def ElementToNode(e):
    p = etree.Element('pool')
    p.attrib['version'] = '1.0.0'

    en = etree.SubElement(p, 'element')
    en.attrib['number'] = str(e.number)
    en.attrib['name'] = e.name
    en.attrib['date'] = '%s through %s' % (
            utils.DateToStr(e.date_created), utils.DateToStr(e.date_expires))
    en.attrib['graphicsdir'] = e.graphics_directory
    en.attrib['version'] = str(e.version)

    for se in e.subelements:
        sen = etree.SubElement(en, 'subelement')
        sen.attrib['designation'] = se.designation
        sen.attrib['name'] = se.description
        sen.attrib['nr_questions'] = str(len(se.question_groups))
        sen.attrib['nr_groups'] = str(len(se.question_groups))

        for g in se.question_groups:
            gn = etree.SubElement(sen, 'group')
            gn.attrib['letter'] = g.designation[2:]
            gn.attrib['description'] = g.description

            for q in g.questions:
                qn = etree.SubElement(gn, 'question')
                qn.attrib['id'] = q.designation[3:]
                qn.attrib['right_answer'] = q.correct_answer
                qn.attrib['text'] = q.text
                if q.figure_designation is not None:
                    qn.attrib['figure'] = q.figure_designation
                if q.is_withdrawn:
                    qn.attrib['withdrawn'] = 'true'

                for a in q.answers:
                    an = etree.SubElement(qn, 'answer')
                    an.attrib['letter'] = a.letter
                    an.attrib['text'] = a.text

    return p


def ElementToXml(e):
    r = ElementToNode(e)
    etree.indent(r, space='\t')
    return etree.tostring(r, xml_declaration=True)
