package net.rten.palmve.utilities;

import androidx.lifecycle.LiveData;
import androidx.lifecycle.Observer;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

public class LiveDataTestUtil {
    /**
     * Helper to get data out of a LiveData object, waiting if necessary.
     * <p>
     * Requires:
     *
     * <pre>{@code
     *     @Rule
     *     public InstantTaskExecutorRule _executor = new InstantTaskExecutorRule();
     * }</pre>
     * <p>
     * To make the test run in the main thread.
     *
     * @param liveData The live data object to load.
     * @param <T>      The type contained in liveData.
     * @return The data contained in liveData.
     */
    public static <T> T getOrAwaitValue(final LiveData<T> liveData) {
        final CountDownLatch latch = new CountDownLatch(1);
        final Observer<T> observer = o -> latch.countDown();
        liveData.observeForever(observer);
        try {
            if (!latch.await(240, TimeUnit.SECONDS)) {
                throw new RuntimeException("timed out waiting for data");
            }
        } catch (InterruptedException e) {
            throw new RuntimeException("await() interrupted");
        } finally {
            liveData.removeObserver(observer);
        }
        return liveData.getValue();
    }
}
