package net.rten.palmve.activities;

import android.os.Bundle;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.FragmentTransaction;
import androidx.lifecycle.LiveData;

import net.rten.palmve.R;
import net.rten.palmve.data.pool.PoolItemWithDescriptionAndDesignation;
import net.rten.palmve.fragments.GenericListFragment;
import net.rten.palmve.fragments.GenericListHeaderFragment;
import net.rten.palmve.models.GenericListModel;
import net.rten.palmve.utilities.GenericListBinder;

import java.util.List;

public abstract class GenericListActivity<
        HV extends PoolItemWithDescriptionAndDesignation,
        IV extends PoolItemWithDescriptionAndDesignation,
        M extends GenericListModel<HV, IV>>
        extends AppCompatActivity
        implements GenericListBinder<HV, IV> {

    static public final String INTENT_POOL_ID = "poolId";

    private M _model;

    abstract protected M makeModel();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.generic_list_activity);
        _model = makeModel();

        if (savedInstanceState == null) {
            FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
            transaction.replace(R.id.genericListActivityHeaderFragment, new GenericListHeaderFragment());
            transaction.replace(R.id.genericListActivityListFragment, new GenericListFragment<HV, IV>());
            transaction.commit();
        }
    }

    @Override
    protected void onStart() {
        super.onStart();

        GenericListHeaderFragment headerFragment =
                (GenericListHeaderFragment) getSupportFragmentManager().findFragmentById(
                        R.id.genericListActivityHeaderFragment);
        if (headerFragment != null) {
            headerFragment.onViewModelReady(_model.getHeaderDataCopy());
        }

        @SuppressWarnings("unchecked") GenericListFragment<HV, IV> listFragment =
                (GenericListFragment<HV, IV>) getSupportFragmentManager().findFragmentById(
                        R.id.genericListActivityListFragment);
        if (listFragment != null) {
            listFragment.onViewModelReady(this);
        }
    }

    @Override
    public LiveData<List<IV>> getListFromModel() {
        return _model.getItemsDataCopy();
    }

    protected int getPoolId() {
        return getIntent().getIntExtra(INTENT_POOL_ID, 0);
    }
}
