package net.rten.palmve.activities;

import android.os.Bundle;

import androidx.appcompat.app.AppCompatActivity;
import androidx.lifecycle.ViewModelProvider;

import net.rten.palmve.LocalPool;
import net.rten.palmve.Preferences;
import net.rten.palmve.R;
import net.rten.palmve.data.pool.Answer;
import net.rten.palmve.fragments.AnswerFragment;
import net.rten.palmve.models.QuestionModel;

abstract class QuestionAndAnswersActivity extends AppCompatActivity {

    final protected QuestionModel.Mode _mode;

    public QuestionAndAnswersActivity(QuestionModel.Mode mode) {
        _mode = mode;

        getSupportFragmentManager().addFragmentOnAttachListener((fragmentManager, fragment) -> {
            Answer.AnswerCode answer = null;
            if (fragment.getId() == R.id.qaActivityFragmentAnswerA) {
                answer = Answer.AnswerCode.A;
            } else if (fragment.getId() == R.id.qaActivityFragmentAnswerB) {
                answer = Answer.AnswerCode.B;
            } else if (fragment.getId() == R.id.qaActivityFragmentAnswerC) {
                answer = Answer.AnswerCode.C;
            } else if (fragment.getId() == R.id.qaActivityFragmentAnswerD) {
                answer = Answer.AnswerCode.D;
            }
            if (answer != null) {
                ((AnswerFragment) fragment).setWhichAnswerAmI(answer);
            }
        });
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.question_and_answers_activity);

        QuestionModel model = new ViewModelProvider(this).get(QuestionModel.class);
        if (!model.isSetUp()) {
            // Copy preferences.
            model.setWrongAnswerBehavior(Preferences.getTestTakingWrongAnswer(this));
            model.setHideWithdrawn(Preferences.getQuestionBrowserHideWithdrawn(this));
            model.setHideAnswerOnBrowse(Preferences.getQuestionBrowserMaskAnswers(this));

            // Set up the mode, pool IDs, give it the DB.
            final LocalPool l = getPool();
            model.setPoolId(l.getLocalPoolId());
            model.setMode(_mode);
            model.setUp(l.getRoomDb());
        }
    }

    abstract protected LocalPool getPool();

    protected int getPoolId() {
        return getIntent().getIntExtra(GenericListActivity.INTENT_POOL_ID, 0);
    }

    protected void setActiveQuestionId(long questionId) {
        QuestionModel model = new ViewModelProvider(this).get(QuestionModel.class);
        model.setPrimaryId(questionId);
    }
}
