/*
 * PalmVE - Android amateur radio test practice program
 * Copyright (C) 2010 Michael A Ballbach
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.rten.palmve.activities;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;

import androidx.appcompat.app.AppCompatActivity;
import androidx.lifecycle.ViewModelProvider;

import net.rten.palmve.LocalPool;
import net.rten.palmve.R;
import net.rten.palmve.models.TestHistoryStatsModel;
import net.rten.palmve.repository.PoolRepo;
import net.rten.palmve.repository.TestHistoryRepo;

import javax.inject.Inject;

import dagger.hilt.android.AndroidEntryPoint;

@AndroidEntryPoint
public class TestHistoryStatsActivity extends AppCompatActivity {

    public static final String INTENT_POOL_NAME = "net.rten.palmve.poolName";
    public static final String INTENT_POOL_DATE = "net.rten.palmve.poolDate";

    private static final String TAG = "TestHistoryStats";

    @Inject
    protected PoolRepo _pools;

    @Inject
    protected TestHistoryRepo _testHistory;

    private TestHistoryStatsModel _model;
    private String _poolName, _poolDateField;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.test_history_stats_activity);
        _poolName = getIntent().getStringExtra(INTENT_POOL_NAME);
        _poolDateField = getIntent().getStringExtra(INTENT_POOL_DATE);

        final LocalPool localPool = _pools.findLocalPool(_poolName, _poolDateField);
        if (localPool == null) {
            Log.w(TAG, "no matching pool for intent");
        } else {
            _model = new ViewModelProvider(this).get(TestHistoryStatsModel.class);
            _model.setUp(_poolName, _poolDateField, localPool.getRoomDb(), _testHistory.getDb());
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.test_history_stats_options, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.testHistoryStatsOptionsClearHistory) {
            _model.asyncDeleteTestHistory();
            return true;
        } else if (item.getItemId() == R.id.testHistoryStatsOptionsViewTests) {
            Intent intent = new Intent(this, TestHistoryListActivity.class);
            intent.setData(getIntent().getData());
            intent.putExtra(TestHistoryListActivity.INTENT_POOL_NAME, _poolName);
            intent.putExtra(TestHistoryListActivity.INTENT_POOL_DATE, _poolDateField);
            startActivity(intent);
            return true;
        } else {
            return super.onOptionsItemSelected(item);
        }
    }
}
