package net.rten.palmve.activities;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;

import androidx.lifecycle.ViewModelProvider;

import net.rten.palmve.LocalPool;
import net.rten.palmve.R;
import net.rten.palmve.data.pool.Answer;
import net.rten.palmve.data.pool.QuestionView;
import net.rten.palmve.models.QuestionModel;
import net.rten.palmve.repository.PoolRepo;

import javax.inject.Inject;

import dagger.hilt.android.AndroidEntryPoint;

@AndroidEntryPoint
public class ViewQuestionAndAnswersActivity extends QuestionAndAnswersActivity {
    public static final String INTENT_QUESTION_ID = "questionId";
    public static final String INTENT_SHOW_PROVIDED_ANSWER = "providedAnswer";

    @Inject
    protected PoolRepo _pools;

    public ViewQuestionAndAnswersActivity() {
        super(QuestionModel.Mode.browsingQuestion);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setActiveQuestionId(getIntent().getLongExtra(INTENT_QUESTION_ID, 0));

        // Observe for title changes.
        QuestionModel model = new ViewModelProvider(this).get(QuestionModel.class);
        model.getHeaderDataCopy().observe(this, question -> setTitle(question.designation));

        // They may have given us a selected answer to highlight, if so, do it.
        Answer.AnswerCode providedAnswer = (Answer.AnswerCode) getIntent().getSerializableExtra(
                INTENT_SHOW_PROVIDED_ANSWER);
        if (providedAnswer != null) {
            model.setSelectedAnswer(providedAnswer);
            model.setHideAnswerOnBrowse(false);
        }

        // And observe to highlight the answer.
        model.getSelectedAnswer().observe(this, answer -> {
            QuestionView question = model.getHeaderDataCopy().getValue();
            if (question != null && answer != null &&
                    model.getHideAnswerOnBrowse().getValue() != null &&
                    model.getHideAnswerOnBrowse().getValue()) {
                if (question.rightAnswer.equals(answer)) {
                    // They selected it right, so just show it.
                    model.setHideAnswerOnBrowse(false);
                } else {
                    // They selected it wrong, and we are not showing it. Ask if they want to
                    // re-try, if not, go ahead and show it.
                    AlertDialog.Builder b = new AlertDialog.Builder(this);
                    DialogInterface.OnClickListener clickListener = (dialog, id) -> {
                        if (id == DialogInterface.BUTTON_POSITIVE) {
                            model.setHideAnswerOnBrowse(false);
                        }
                    };

                    b.setMessage(getText(R.string.Browse_Answers_ShowAnswerPrompt));
                    b.setCancelable(false);
                    b.setPositiveButton(R.string.Standard_Yes, clickListener);
                    b.setNegativeButton(R.string.Standard_No, clickListener);
                    b.show();
                }
            }
        });
    }

    @Override
    protected LocalPool getPool() {
        return _pools.getLocalPoolById(getPoolId());
    }
}
