/*
 * PalmVE - Android amateur radio test practice program
 * Copyright (C) 2010 Michael A Ballbach
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.rten.palmve.asynctasks;

import android.util.Xml;

import net.rten.palmve.RemotePool;
import net.rten.palmve.utilities.FileCopierInputStream;
import net.rten.palmve.utilities.RemotePoolXmlParser;

import org.xml.sax.SAXException;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;

/**
 * Download the available remote pools.
 */
public class FetchRemotePoolListAsyncTask {
    /**
     * Actually run the fetch.
     */
    static public ArrayList<RemotePool> run(URL url, File xmlCacheFile) throws IOException, SAXException {
        RemotePoolXmlParser parser = new RemotePoolXmlParser();
        File cachedDownloadFile = new File(xmlCacheFile + "~");
        FileOutputStream cachedOutputStream = new FileOutputStream(cachedDownloadFile);

        // connect and download/copy:
        URLConnection connection = url.openConnection();
        InputStream inputStream = new FileCopierInputStream(connection
                .getInputStream(), cachedOutputStream);
        Xml.parse(inputStream, Xml.Encoding.US_ASCII, parser);

        // if the parsing was good, rename and accept the cache
        if (!cachedDownloadFile.renameTo(xmlCacheFile)) {
            throw new RuntimeException("renaming operation failed: " + xmlCacheFile);
        }
        return parser.getRemotePools();
    }
}
