package net.rten.palmve.data.pool;

import androidx.lifecycle.LiveData;
import androidx.room.Dao;
import androidx.room.Query;

import java.util.List;

@Dao
public interface QuestionDao {
    @Query("SELECT Question.* FROM Question " +
            "INNER JOIN QuestionGroup ON QuestionGroup._id = Question.questiongroup " +
            "INNER JOIN SubElement ON SubElement._id = QuestionGroup.subelement " +
            "WHERE SubElement.designation = :subElement AND " +
            "QuestionGroup.letter = :questionGroup AND " +
            "Question.tag = :questionTag")
    Question findQuestionByIdentifiers(String subElement, String questionGroup, String questionTag);

    @Query("SELECT description FROM FTS_Question WHERE rowid = :questionId")
    String getTextForQuestionId(long questionId);

    @Query("SELECT * FROM QuestionView")
    List<QuestionView> questions();

    @Query("SELECT * FROM QuestionView WHERE questionGroupId = :questionGroupId")
    LiveData<List<QuestionView>> questionsByQuestionGroupId(long questionGroupId);

    @Query("SELECT * FROM QuestionView WHERE questionGroupId = :questionGroupId AND NOT withdrawn")
    LiveData<List<QuestionView>> questionsByQuestionGroupIdHideWithdrawn(long questionGroupId);

    @Query("SELECT * FROM AnswerView WHERE subElement = :subElement AND questionGroup = :questionGroup AND questionTag = :questionTag")
    List<AnswerView> getAnswers(String subElement, String questionGroup, String questionTag);

    @Query("SELECT * FROM QuestionView WHERE id = :questionId")
    QuestionView getQuestionById(long questionId);

    @Query("SELECT * FROM QuestionView WHERE id = :questionId")
    LiveData<QuestionView> getQuestionByIdLiveData(long questionId);

    @Query("SELECT * FROM AnswerView WHERE questionId = :questionId")
    LiveData<List<AnswerView>> getAnswersByQuestionIdLiveData(long questionId);

    @Query("SELECT * FROM QuestionView WHERE description MATCH :search")
    List<QuestionView> searchQuestions(String search);

    @Query("SELECT * FROM AnswerView WHERE answerText MATCH :search")
    List<AnswerView> searchAnswers(String search);

    @Query("SELECT * FROM QuestionView WHERE description MATCH :search")
    LiveData<List<QuestionView>> searchQuestionsLiveData(String search);

    @Query("SELECT * FROM QuestionViewForAnswerSearch WHERE answerText MATCH :search")
    LiveData<List<QuestionViewForAnswerSearch>> searchQuestionAnswersLiveData(String search);

    @Query("SELECT description FROM QuestionView WHERE id = :questionId")
    LiveData<String> getQuestionDescription(long questionId);
}