package net.rten.palmve.data.test;

import androidx.lifecycle.LiveData;
import androidx.room.Dao;
import androidx.room.Query;

import net.rten.palmve.data.pool.Answer;

import java.util.Calendar;
import java.util.List;

@Dao
public interface TestHistoryDao {
    @Query("SELECT * FROM TestHistory")
    LiveData<List<TestHistory>> getAllTests();

    @Query("SELECT * FROM TestQuestions WHERE testid = :testId ORDER BY qindex ASC")
    List<TestQuestions> getQuestionsInTest(long testId);

    @Query("SELECT * FROM TestQuestions WHERE testid = :testId ORDER BY qindex ASC")
    LiveData<List<TestQuestions>> getQuestionsInTestLiveData(long testId);

    @Query("SELECT COUNT(_id) AS testCount, state AS testState FROM TestHistory " +
            "WHERE element = :elementName AND elementdatefield = :elementDate " +
            "AND datecompleted IS NOT NULL GROUP BY state")
    LiveData<List<TestCount>> getTestCounts(String elementName, String elementDate);

    @Query("SELECT COUNT(_id) AS testCount, state AS testState FROM TestHistory " +
            "WHERE element = :elementName AND elementdatefield = :elementDate " +
            "AND datecompleted >= :newerThan GROUP BY state")
    LiveData<List<TestCount>> getTestCountsNewerThan(String elementName, String elementDate,
                                                     Calendar newerThan);

    @Query("DELETE FROM TestHistory WHERE element = :elementName AND elementdatefield = :elementDate")
    void deleteTestHistory(String elementName, String elementDate);

    @Query("SELECT * FROM TestHistory WHERE element = :elementName AND elementdatefield = :elementDate " +
            "AND state <> 1 ORDER BY datecompleted DESC")
    LiveData<List<TestHistory>> getTestHistory(String elementName, String elementDate);

    @Query("SELECT * FROM TestHistory WHERE _id = :testId")
    TestHistory getTestInfoById(long testId);

    @Query("SELECT * FROM TestHistory WHERE _id = :testId")
    LiveData<TestHistory> getTestInfoByIdLiveData(long testId);

    @Query("INSERT INTO TestHistory (datestarted, element, elementversion, elementdatefield, " +
            "state, qcount) VALUES (:dateStarted, :elementName, :elementVersion, :elementDateField, " +
            "1, :questionCount)")
    long insertTest(Calendar dateStarted, String elementName, int elementVersion,
                    String elementDateField, int questionCount);

    @Query("INSERT INTO TestQuestions (testid, questionid, questiondesignation, qgroupid, " +
            "correctanswer, qindex, nextid) VALUES (:testId, :questionId, :questionDesignation, " +
            ":questionGroupId, :correctAnswer, :questionIndex, :nextQuestionId)")
    long insertQuestion(long testId, long questionId, String questionDesignation,
                        long questionGroupId, Answer.AnswerCode correctAnswer, int questionIndex,
                        Long nextQuestionId);

    @Query("UPDATE TestHistory SET firstquestion = :firstQuestionId WHERE _id = :testId")
    void setTestFirstQuestion(long testId, long firstQuestionId);
}
