package net.rten.palmve.fragments;

import android.graphics.LightingColorFilter;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import net.rten.palmve.R;
import net.rten.palmve.data.pool.Answer;
import net.rten.palmve.data.pool.AnswerView;
import net.rten.palmve.data.pool.QuestionView;
import net.rten.palmve.models.QuestionModel;

import java.util.List;

public class AnswerFragment extends Fragment {
    private static final String TAG = "AnswerFragment";
    private TextView _text;
    private Button _button;
    private Answer.AnswerCode _whichAnswerAmI;
    private QuestionModel _model;

    static public int getAnswerLetterResource(Answer.AnswerCode code) {
        switch (code) {
            case A:
                return R.string.Answer_A;
            case B:
                return R.string.Answer_B;
            case C:
                return R.string.Answer_C;
            case D:
                return R.string.Answer_D;
        }
        throw new RuntimeException("invalid answer code");
    }

    public void setWhichAnswerAmI(Answer.AnswerCode to) {
        _whichAnswerAmI = to;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.answer_fragment, container, false);
        rootView.setTag(TAG);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        _text = view.findViewById(R.id.answerFragmentLabel);
        _button = view.findViewById(R.id.answerFragmentButton);

        _model = new ViewModelProvider(requireActivity()).get(QuestionModel.class);
        _model.getItemsDataCopy().observe(getViewLifecycleOwner(), answers -> {
            AnswerView answerView = getOurAnswerView(answers);
            _text.setText(answerView.answerText);
            _button.setText(getString(getOurAnswerLetterResource()));
            decorate();
        });
        _model.getHeaderDataCopy().observe(getViewLifecycleOwner(), question -> decorate());
        _model.getHideAnswerOnBrowse().observe(getViewLifecycleOwner(), hide -> decorate());
        _model.getSelectedAnswer().observe(getViewLifecycleOwner(), answer -> decorate());
        _button.setOnClickListener(clicked -> _model.setSelectedAnswer(_whichAnswerAmI));
    }

    private void decorate() {
        Boolean hidingAnswer = _model.getHideAnswerOnBrowse().getValue();
        QuestionView question = _model.getHeaderDataCopy().getValue();
        Answer.AnswerCode providedAnswer = _model.getSelectedAnswer().getValue();
        if (hidingAnswer != null && question != null) {
            if (!hidingAnswer) {
                if (_whichAnswerAmI.equals(question.rightAnswer)) {
                    // We are the correct answer button.
                    _button.getBackground().setColorFilter(new LightingColorFilter(
                            ContextCompat.getColor(requireContext(), R.color.ColorRightAnswerHighlight),
                            0));
                    _button.setText(getString(R.string.Browse_Answers_RightAnswerTag,
                            getString(getOurAnswerLetterResource())));
                } else if (_whichAnswerAmI.equals(providedAnswer)) {
                    // They provided a wrong answer, and we are that button.
                    _button.getBackground().setColorFilter(new LightingColorFilter(
                            ContextCompat.getColor(requireContext(), R.color.ColorWrongAnswerHighlight),
                            0));
                    _button.setText(getString(R.string.Browse_Answers_WrongAnswerTag,
                            getString(getOurAnswerLetterResource())));
                } else {
                    // Clear any decoration.
                    _button.getBackground().setColorFilter(null);
                    _button.setText(getString(getOurAnswerLetterResource()));
                }
            }
        }
    }

    private int getOurAnswerLetterResource() {
        return getAnswerLetterResource(_whichAnswerAmI);
    }

    @NonNull
    private AnswerView getOurAnswerView(List<AnswerView> viewData) {
        switch (_whichAnswerAmI) {
            case A:
                return viewData.get(0);
            case B:
                return viewData.get(1);
            case C:
                return viewData.get(2);
            case D:
                return viewData.get(3);
        }
        throw new RuntimeException("invalid _whichAnswerAmI");
    }
}