package net.rten.palmve.fragments;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.DiffUtil;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.ListAdapter;
import androidx.recyclerview.widget.RecyclerView;

import net.rten.palmve.R;
import net.rten.palmve.activities.GenericListActivity;
import net.rten.palmve.activities.ViewQuestionAndAnswersActivity;
import net.rten.palmve.data.pool.QuestionView;
import net.rten.palmve.models.PoolSearchResultsModel;
import net.rten.palmve.utilities.PoolItemDiffCallback;

public class PoolSearchResultsFragment extends Fragment {
    private static final String TAG = "SearchResultsFragment";

    private final DiffUtil.ItemCallback<QuestionView> _diffCallback = new PoolItemDiffCallback<>();

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.search_results, container, false);
        rootView.setTag(TAG);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        final ItemAdapter adapter = new ItemAdapter();
        final TextView titleView = view.findViewById(R.id.searchResultsTag);

        RecyclerView recyclerView = view.findViewById(R.id.searchResultsList);
        LinearLayoutManager layoutManager = new LinearLayoutManager(getActivity());
        recyclerView.setLayoutManager(layoutManager);
        recyclerView.setAdapter(adapter);
        recyclerView.addItemDecoration(new DividerItemDecoration(
                recyclerView.getContext(), DividerItemDecoration.VERTICAL));

        PoolSearchResultsModel model = new ViewModelProvider(requireActivity()).get(
                PoolSearchResultsModel.class);
        model.getSearchResults().observe(getViewLifecycleOwner(), results -> {
            titleView.setText(getResources().getQuantityString(
                    R.plurals.Search_ResultCount, results.size(), results.size()));
            adapter.submitList(results);
        });
    }

    private void onItemIdClicked(long questionIdWithElemInHighOrderBits) {
        final int element = (int)(questionIdWithElemInHighOrderBits >> 16);
        final long questionId = questionIdWithElemInHighOrderBits & 0xffff;

        Intent intent = new Intent(getActivity(), ViewQuestionAndAnswersActivity.class);
        intent.putExtra(GenericListActivity.INTENT_POOL_ID, element);
        intent.putExtra(ViewQuestionAndAnswersActivity.INTENT_QUESTION_ID, questionId);
        startActivity(intent);
    }

    static private class ItemViewHolder extends RecyclerView.ViewHolder {
        private final TextView _tag;
        private final TextView _description;

        public ItemViewHolder(View view) {
            super(view);
            _tag = view.findViewById(R.id.searchResultItemQuestionId);
            _description = view.findViewById(R.id.searchResultQuestionExcerpt);
        }
    }

    private class ItemAdapter extends ListAdapter<QuestionView, ItemViewHolder> {
        public ItemAdapter() {
            super(_diffCallback);
        }

        @NonNull
        @Override
        public ItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View itemView = LayoutInflater.from(parent.getContext()).inflate(
                    R.layout.search_results_item, parent, false);
            return new ItemViewHolder(itemView);
        }

        @Override
        public void onBindViewHolder(@NonNull ItemViewHolder holder, int position) {
            QuestionView item = getItem(position);
            holder._tag.setText(item.designation);
            holder._description.setText(item.description);
            holder.itemView.setOnClickListener(
                    clickedView -> onItemIdClicked(getItem(position).id));
        }
    }
}