package net.rten.palmve.fragments;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import net.rten.palmve.R;
import net.rten.palmve.data.test.TestHistory;
import net.rten.palmve.data.test.TestState;
import net.rten.palmve.models.TestQuestionData;
import net.rten.palmve.models.TestSummaryModel;

import java.util.List;

public class TestResultsHeaderFragment extends Fragment {
    private static final String TAG = "TestRsltsHdrFrag";

    private ImageView _summaryIcon;
    private TextView _summaryTag;
    private TextView _summaryPercent;

    private TestHistory _testHistory;
    private List<TestQuestionData> _testQuestions;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.test_results_header_fragment, container, false);
        rootView.setTag(TAG);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        _summaryIcon = view.findViewById(R.id.testResultsHeaderIcon);
        _summaryTag = view.findViewById(R.id.testResultsHeaderTag);
        _summaryPercent = view.findViewById(R.id.testResultsHeaderPercentage);

        TestSummaryModel model = new ViewModelProvider(requireActivity()).get(TestSummaryModel.class);
        model.getTestInfo().observe(getViewLifecycleOwner(), testInfo -> {
            _testHistory = testInfo;
            updateUi();
        });
        model.getTestQuestions().observe(getViewLifecycleOwner(), testQuestions -> {
            _testQuestions = testQuestions;
            updateUi();
        });
    }

    private void updateUi() {
        if (_testHistory == null) {
            return;
        }

        final boolean passed = _testHistory.testState.equals(TestState.PASSED);
        _summaryIcon.setImageResource(passed
                ? R.drawable.ic_baseline_check_circle_outline_green_24
                : R.drawable.ic_outline_cancel_red_24);
        _summaryIcon.setContentDescription(getString(
                passed ? R.string.Standard_Passed : R.string.Standard_Failed));
        _summaryTag.setText(passed ? R.string.TestSummary_Passed : R.string.TestSummary_Failed);

        // We might not know the total question counts, yet.
        String percentString;
        if (_testQuestions != null) {
            int correct = 0;
            for (TestQuestionData q : _testQuestions) {
                if (q.testQuestion.correctAnswer != null &&
                        q.testQuestion.correctAnswer.equals(q.testQuestion.providedAnswer)) {
                    ++correct;
                }
            }
            percentString = String.format(getString(R.string.TestSummary_GradeNumbers),
                    _testHistory.testScorePercent, correct, _testQuestions.size());
        } else {
            percentString = String.format(getString(R.string.TestSummary_GradeNumbersWithoutCounts),
                    _testHistory.testScorePercent);
        }
        _summaryPercent.setText(percentString);
    }
}
