package net.rten.palmve.fragments;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.DiffUtil;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.ListAdapter;
import androidx.recyclerview.widget.RecyclerView;

import net.rten.palmve.R;
import net.rten.palmve.activities.GenericListActivity;
import net.rten.palmve.activities.ViewQuestionAndAnswersActivity;
import net.rten.palmve.data.Converters;
import net.rten.palmve.models.TestQuestionData;
import net.rten.palmve.models.TestSummaryModel;

public class TestResultsQuestionsFragment extends Fragment {
    private static final String TAG = "TestRsltsQFrag";
    private final DiffUtil.ItemCallback<TestQuestionData> _diffCallback = new DiffUtil.ItemCallback<TestQuestionData>() {
        @Override
        public boolean areItemsTheSame(@NonNull TestQuestionData oldItem, @NonNull TestQuestionData newItem) {
            return oldItem.testQuestion.id.equals(newItem.testQuestion.id);
        }

        @Override
        public boolean areContentsTheSame(@NonNull TestQuestionData oldItem, @NonNull TestQuestionData newItem) {
            if (oldItem.poolQuestion == null && newItem.poolQuestion == null) {
                return true;
            } else if (oldItem.poolQuestion == null || newItem.poolQuestion == null) {
                return false;
            } else {
                return oldItem.poolQuestion.description.equals(newItem.poolQuestion.description);
            }
        }
    };
    private final ItemAdapter _adapter = new ItemAdapter();
    private TestSummaryModel _model;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.recycler_view_fragment, container, false);
        rootView.setTag(TAG);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        _model = new ViewModelProvider(requireActivity()).get(TestSummaryModel.class);

        // setup the recycler view
        LinearLayoutManager layoutManager = new LinearLayoutManager(requireActivity());
        RecyclerView list = view.findViewById(R.id.recyclerViewList);
        list.setLayoutManager(layoutManager);
        list.setAdapter(_adapter);
        list.addItemDecoration(new DividerItemDecoration(
                list.getContext(), DividerItemDecoration.VERTICAL));

        // the first observe() seeds the list of things, the second is required to load the
        // question or question group strings, as these come after the initial list is ready.
        _model.getTestQuestions().observe(getViewLifecycleOwner(), _adapter::submitList);
        _model.getTestItemUpdated().observe(getViewLifecycleOwner(), _adapter::notifyItemChanged);
    }

    protected void onItemIdClicked(TestQuestionData test) {
        Intent intent = new Intent(getActivity(), ViewQuestionAndAnswersActivity.class);
        intent.putExtra(ViewQuestionAndAnswersActivity.INTENT_QUESTION_ID, test.poolQuestion.id);
        intent.putExtra(ViewQuestionAndAnswersActivity.INTENT_SHOW_PROVIDED_ANSWER,
                test.testQuestion.providedAnswer);
        intent.putExtra(GenericListActivity.INTENT_POOL_ID, _model.getPoolId());
        startActivity(intent);
    }

    static private class ItemViewHolder extends RecyclerView.ViewHolder {
        private final ImageView icon;
        private final TextView questionId;
        private final TextView questionExcerpt;
        private final TextView youSaid;
        private final TextView right;

        public ItemViewHolder(View view) {
            super(view);
            icon = view.findViewById(R.id.testSummaryItemIcon);
            questionId = view.findViewById(R.id.testSummaryItemQuestionId);
            questionExcerpt = view.findViewById(R.id.testSummaryItemQuestionExcerpt);
            youSaid = view.findViewById(R.id.testSummaryItemYouSaidValue);
            right = view.findViewById(R.id.testSummaryItemRightValue);
        }
    }

    private class ItemAdapter extends ListAdapter<TestQuestionData, ItemViewHolder> {
        protected ItemAdapter() {
            super(_diffCallback);
        }

        @NonNull
        @Override
        public ItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View itemView = LayoutInflater.from(parent.getContext()).inflate(
                    R.layout.test_results_question_list_item, parent, false);
            return new ItemViewHolder(itemView);
        }

        @Override
        public void onBindViewHolder(@NonNull ItemViewHolder holder, int position) {
            final TestQuestionData item = getItem(position);

            final boolean correct =
                    item.testQuestion.providedAnswer.equals(item.testQuestion.correctAnswer);
            holder.icon.setImageResource(correct
                    ? R.drawable.ic_baseline_check_circle_outline_green_24
                    : R.drawable.ic_outline_cancel_red_24);
            holder.icon.setContentDescription(getString(correct
                    ? R.string.Standard_Correct
                    : R.string.Standard_Incorrect));

            Converters c = new Converters();
            holder.youSaid.setText(c.stringFromAnswerCode(item.testQuestion.providedAnswer));
            holder.right.setText(c.stringFromAnswerCode(item.testQuestion.correctAnswer));

            if (item.poolQuestion != null) {
                holder.questionId.setText(item.poolQuestion.designation);
                holder.questionExcerpt.setText(item.poolQuestion.description);
            }

            holder.itemView.setOnClickListener(clickedView -> onItemIdClicked(getItem(position)));
        }
    }
}
