package net.rten.palmve.models;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;

import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModel;

import net.rten.palmve.data.pool.Figure;
import net.rten.palmve.data.pool.FigureView;
import net.rten.palmve.data.pool.PoolDatabase;

public class FigureModel extends ViewModel {
    private final MutableLiveData<Bitmap> _figureBmpCopy = new MutableLiveData<>();
    private final CopyFigureData _copy = new CopyFigureData();
    private LiveData<Figure> _figure;
    private MutableLiveData<FigureView> _figureCopy = new MutableLiveData<>();
    private long _figureId = -1;

    public LiveData<FigureView> getFigureViewCopy() {
        return _figureCopy;
    }

    public LiveData<Bitmap> getFigureBmpCopy() {
        return _figureBmpCopy;
    }

    protected long getFigureId() {
        if (_figureId < 0) {
            throw new RuntimeException("call setFigureId() first");
        }
        return _figureId;
    }

    public void setFigureId(long id) {
        if (_figureId < 0) {
            _figureId = id;
        } else if (id != _figureId) {
            throw new RuntimeException("unexpected primary ID change");
        }
    }

    public void setUp(PoolDatabase poolDb) {
        if (_figure == null) {
            _figure = poolDb.figureDao().getFigureByIdLiveData(getFigureId());
            _figure.observeForever(_copy);
        }
    }

    @Override
    public void onCleared() {
        if (_figure != null) {
            _figure.removeObserver(_copy);
            _figureCopy = null;
        }
    }

    private class CopyFigureData implements Observer<Figure> {
        @Override
        public void onChanged(Figure item) {
            if (item != null) {
                FigureView fv = new FigureView();
                fv.id = item.id;
                fv.designation = item.name;
                _figureCopy.setValue(fv);

                Bitmap image = BitmapFactory.decodeByteArray(
                        item.data, 0, item.data.length);
                _figureBmpCopy.setValue(image);
            }
        }
    }
}
