package net.rten.palmve.models;

import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModel;

import net.rten.palmve.data.pool.PoolDatabase;
import net.rten.palmve.data.pool.PoolItemWithDescriptionAndDesignation;
import net.rten.palmve.data.pool.PoolItemWithId;

import java.util.List;

public abstract class GenericListModel<
        HV extends PoolItemWithDescriptionAndDesignation,
        IV extends PoolItemWithId>
        extends ViewModel {
    private final MutableLiveData<HV> _headerItemCopy = new MutableLiveData<>();
    private final MutableLiveData<List<IV>> _itemsCopy = new MutableLiveData<>();
    private LiveData<HV> _header;
    private LiveData<List<IV>> _items;
    private final CopyHeaderItem _copyHeaderItem = new CopyHeaderItem();
    private final CopyItems _copyItems = new CopyItems();
    private long _primaryId = -1;
    private PoolDatabase _poolDb;

    protected long getPrimaryId() {
        if (_primaryId < 0) {
            throw new RuntimeException("call setPrimaryId() first");
        }
        return _primaryId;
    }

    public void setPrimaryId(long id) {
        _primaryId = id;

        // If we have a DB already, we can go ahead and load the data.
        if (_poolDb != null) {
            onCleared();
            _header = getHeaderDataFromDao(_poolDb);
            _header.observeForever(_copyHeaderItem);
            _items = getItemDataFromDao(_poolDb);
            _items.observeForever(_copyItems);
        }
    }

    protected abstract LiveData<HV> getHeaderDataFromDao(PoolDatabase poolDb);

    protected abstract LiveData<List<IV>> getItemDataFromDao(PoolDatabase poolDb);

    public void setUp(PoolDatabase poolDb) {
        _poolDb = poolDb;

        // If we have a question ID already, re-invoke setPrimaryId() to start data loading.
        if (_primaryId >= 0) {
            setPrimaryId(_primaryId);
        }
    }

    public boolean isSetUp() {
        return _poolDb != null;
    }

    public LiveData<HV> getHeaderDataCopy() {
        return _headerItemCopy;
    }

    public LiveData<List<IV>> getItemsDataCopy() {
        return _itemsCopy;
    }

    @Override
    public void onCleared() {
        if (_header != null) {
            _header.removeObserver(_copyHeaderItem);
            _header = null;
        }
        if (_items != null) {
            _items.removeObserver(_copyItems);
            _items = null;
        }
    }

    private class CopyHeaderItem implements Observer<HV> {
        @Override
        public void onChanged(HV item) {
            _headerItemCopy.setValue(item);
        }
    }

    private class CopyItems implements Observer<List<IV>> {
        @Override
        public void onChanged(List<IV> items) {
            _itemsCopy.setValue(items);
        }
    }
}