package net.rten.palmve.repository;

import android.content.Context;

import androidx.annotation.VisibleForTesting;

import net.rten.palmve.data.test.TestDatabase;
import net.rten.palmve.data.test.TestDatabaseOpener;

import javax.inject.Singleton;

import dagger.Module;
import dagger.Provides;
import dagger.hilt.InstallIn;
import dagger.hilt.android.qualifiers.ApplicationContext;
import dagger.hilt.components.SingletonComponent;

@Module
@InstallIn(SingletonComponent.class)
public class TestHistoryRepo {
    final private Context _context;
    final private String _fileName;
    private TestDatabase _historyDb;

    public TestHistoryRepo(Context appContext) {
        this(appContext, null);
    }

    /**
     * This is provided so that unit tests can override the location of the database.
     */
    @VisibleForTesting
    public TestHistoryRepo(Context appContext, String fileName) {
        _context = appContext;
        _fileName = fileName;
        reset();
    }

    @Singleton
    @Provides
    public static TestHistoryRepo provideTestHistoryRepo(@ApplicationContext Context appContext) {
        return new TestHistoryRepo(appContext);
    }

    public TestDatabase getDb() {
        return _historyDb;
    }

    @VisibleForTesting
    public void close() {
        if (_historyDb != null) {
            _historyDb.close();
            _historyDb = null;
        }
    }

    /**
     * This is provided so that unit tests can reset the DB between tests.
     */
    @VisibleForTesting
    public void reset() {
        close();
        if (_fileName != null) {
            _historyDb = TestDatabaseOpener.openFile(_context, _fileName);
        } else {
            _historyDb = TestDatabaseOpener.open(_context);
        }
    }
}
