package net.rten.palmve.utilities;

import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.Observer;

/**
 * Maintains a LiveData object. Will observe a master version, and copy its value. Useful when
 * you need a LiveData before the master LiveData object might be ready (for example, if you have
 * to wait for a service to get the master data).
 *
 * @param <I> The type of LiveData
 */
public class CopyObserver<I> implements Observer<I> {
    private final MutableLiveData<I> _copy = new MutableLiveData<>();
    private LiveData<I> _master;

    public LiveData<I> getCopy() {
        return _copy;
    }

    public void startObservingMaster(LiveData<I> master) {
        if (_master != null) {
            stopObservingMaster();
        }
        _master = master;
        _master.observeForever(this);
    }

    public void stopObservingMaster() {
        if (_master != null) {
            _master.removeObserver(this);
            _master = null;
        }
    }

    @Override
    public void onChanged(I i) {
        _copy.setValue(i);
    }
}
